unit umainform;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.Objects,
  FMX.ImgList,

  UDataModule;

type
  TDraggableImage = class(TImage)
    constructor Create(AOwner: TComponent; X, Y: integer; ABitmap: TBitmap); overload;
  end;

  TForm1 = class(TForm)
    imgBoard: TImage;
    procedure FormCreate(Sender: TObject);
    procedure imgAnyPieceMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure imgAnyPieceMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure imgBoardMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Single);
    procedure imgAnyPieceMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Single);
  private
    { Dclarations prives }
  public
    { Dclarations publiques }
    FX, FY: single;
    FImages: array[0..31] of TDraggableImage;
    FCaptured: TImage;
  end;

var
  Form1: TForm1;

const
  tailleCase = 80;
  tailleBordure = 20;

implementation

{$R *.fmx}

constructor TDraggableImage.Create(AOwner: TComponent; X, Y: integer; ABitmap: TBitmap);
begin
  inherited Create(AOwner);
  Parent := Form1;
  Position.X := X;
  Position.Y := Y;
  Width := ABitmap.Width;
  Height := ABitmap.Height;
  Bitmap := ABitmap;
  Cursor := crHandPoint;
  onMouseDown := Form1.imgAnyPieceMouseDown;
  onMouseMove := Form1.imgAnyPieceMouseMove;
  onMouseUp := Form1.imgAnyPieceMouseUp;
end;

procedure TForm1.FormCreate(Sender: TObject);
type
  TData = record
    x, y, piecetype: integer;
  end;
const
  BLACK: array[0..15] of TData = (
    (x: 020; y: 020; piecetype: 3),
    (x: 100; y: 020; piecetype: 1),
    (x: 180; y: 020; piecetype: 2),
    (x: 260; y: 020; piecetype: 4),
    (x: 340; y: 020; piecetype: 5),
    (x: 420; y: 020; piecetype: 2),
    (x: 500; y: 020; piecetype: 1),
    (x: 580; y: 020; piecetype: 3),
    (x: 020; y: 100; piecetype: 0),
    (x: 100; y: 100; piecetype: 0),
    (x: 180; y: 100; piecetype: 0),
    (x: 260; y: 100; piecetype: 0),
    (x: 340; y: 100; piecetype: 0),
    (x: 420; y: 100; piecetype: 0),
    (x: 500; y: 100; piecetype: 0),
    (x: 580; y: 100; piecetype: 0)
  );
  WHITE: array[0..15] of TData = (
    (x: 020; y: 580; piecetype: 3),
    (x: 100; y: 580; piecetype: 1),
    (x: 180; y: 580; piecetype: 2),
    (x: 260; y: 580; piecetype: 4),
    (x: 340; y: 580; piecetype: 5),
    (x: 420; y: 580; piecetype: 2),
    (x: 500; y: 580; piecetype: 1),
    (x: 580; y: 580; piecetype: 3),
    (x: 020; y: 500; piecetype: 0),
    (x: 100; y: 500; piecetype: 0),
    (x: 180; y: 500; piecetype: 0),
    (x: 260; y: 500; piecetype: 0),
    (x: 340; y: 500; piecetype: 0),
    (x: 420; y: 500; piecetype: 0),
    (x: 500; y: 500; piecetype: 0),
    (x: 580; y: 500; piecetype: 0)
  );
var
  s: TSizeF;
  i: integer;
begin
  FCaptured := nil;
  s.Create(80, 80);
  for i := 0 to 15 do
  begin
    FImages[i] := TDraggableImage.Create(Self, BLACK[i].x, BLACK[i].y, DataModule2.ImageList2.Bitmap(s, BLACK[i].piecetype));
    FImages[i] := TDraggableImage.Create(Self, WHITE[i].x, WHITE[i].y, DataModule2.ImageList1.Bitmap(s, WHITE[i].piecetype));
  end;
end;

procedure TForm1.imgBoardMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Single);
begin
  if Assigned(FCaptured) then
  begin
    FCaptured.Position.X := X - FX;
    FCaptured.Position.Y := Y - FY;
  end;
end;

procedure TForm1.imgAnyPieceMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
  FCaptured := sender as TImage;
  (sender as TImage).BringToFront;
  FX := X;
  FY := Y;
end;

procedure TForm1.imgAnyPieceMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Single);
begin
  if Assigned(FCaptured) then
  begin
    with sender as TImage do
    begin
      Position.X := X + Position.X - FX;
      Position.Y := Y + Position.Y - FY;
    end;
  end;
end;

procedure TForm1.imgAnyPieceMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
var
  caseX, caseY : integer;
  caseCentreX, caseCentreY : single;
begin
  FCaptured := nil;
  with (sender as TImage) do
  begin
    caseX := Trunc((position.X + X-tailleBordure) / tailleCase);
    caseY := Trunc((position.Y + Y-tailleBordure) / tailleCase);
    caseCentreX := tailleBordure + (tailleCase * caseX);
    caseCentreY := tailleBordure + (tailleCase * caseY);
    AnimateFloat('Position.X', caseCentreX);
    AnimateFloat('Position.Y', caseCentreY);
  end;
end;

end.
